# -*- coding:utf-8 -*-

from foxrelax.common.code import (Code, code_to_message)
from foxrelax.common.exceptions import (RequestError, ResponseError)
from foxrelax.common.util import (root_path, version, install_requires, md5,
                                  gen_timestamp, gen_nonce_str, gen_uuid, sign)

from foxrelax.client import api_client
from foxrelax.config import auth

from foxrelax.api import (
    echo, trade_calendar, area, exchange_info, stock_info, valuation, dividend,
    capital_change, adj_factor, bar, index_info, index_daily, index_weekly,
    index_monthly, index_bar, stock_trade_monthly, manufacturing_pmi_monthly,
    ppi_monthly, cci_monthly, north_top10, north_money, south_money,
    north_money_realtime, south_money_realtime, rise_fall_realtime,
    limit_rise_fall_realtime, normalize_symbol, normalize_stock_symbol,
    normalize_index_symbol, news_flash, stock_daily, stock_weekly,
    stock_monthly, stock_bar, balance_sheet, income, cash_flow,
    industry_industry, industry_concept, industry_region, industry,
    industry_stock_industry, industry_stock_concept, industry_stock_region,
    industry_stock, stock_industry_industry, stock_industry_concept,
    stock_industry_region, stock_industry, money_supply, reserve_ratio,
    gold_foreign_reserve_monthly, stock_suspend, deposit_rate, loan_rate, cpi,
    cpi_item)



# -*- coding:utf-8 -*-
import threading
import sys
from enum import Enum, unique

API_URL = 'http://api.foxrelax.com'
API_VERSION = '1.0'
IS_PY3 = (sys.version_info.major >= 3)
THREAD_LOCAL = threading.local()
DEFAULT_TIMEOUT = 15  # seconds


@unique
class SignType(Enum):
    MD5 = 'md5'


@unique
class Charset(Enum):
    UTF8 = 'utf-8'


@unique
class Lang(Enum):
    EN = 'en'  # 英文



# -*- coding:utf-8 -*-
from functools import partial
import requests
from foxrelax.config import auth, Config
from foxrelax.common.request import Request
from foxrelax.common.response import Response


def api_client(app_id=None, app_secret=None, api_url=None):
    auth(app_id, app_secret, api_url)
    return ApiClient()


class ApiClient:
    """
    HTTP Client
    """
    def __init__(self):
        self._config = Config()
        self.__headers = {
            'Content-Type':
            'application/json; charset={charset}'.format(
                charset=self._config.charset),
            'User-Agent':
            'foxrelax-client-{version}'.format(
                version=self._config.client_version)
        }

    @property
    def config(self):
        return self._config

    def query(self, method, **kwargs):
        trace = kwargs.pop('trace', False)

        req = Request(method, kwargs)
        res = requests.post(self.config.api_url,
                            json=req.to_dict(),
                            timeout=self.config.timeout)
        resp = Response(res.text)

        if trace:
            print('REQ=>{0}'.format(req))
            print('RESP=>{0}'.format(resp))

        return resp

    def __getattr__(self, name):
        return partial(self.query, name)



# -*- coding:utf-8 -*-

from foxrelax.common.consts import API_URL, API_VERSION, SignType, Charset, Lang, DEFAULT_TIMEOUT
from foxrelax.common.util import version, gen_uuid


def auth(app_id=None, app_secret=None, api_url=None):
    """
    用户认证
    """

    config = Config()
    if app_id is None:
        app_id = gen_uuid()
    if app_secret is None:
        app_secret = gen_uuid()

    if app_id:
        config.app_id = app_id

    if app_secret:
        config.app_secret = app_secret

    if api_url:
        config.api_url = api_url


class Config:
    """
    单例模式的全局配置
    """

    def __new__(cls, *args, **kwargs):
        return cls.init(*args, **kwargs)

    @classmethod
    def init(cls):
        """
        初始化全局配置
        """

        if not hasattr(cls, '_instance'):
            cls._instance = super(Config, cls).__new__(cls)

            Config._api_url = API_URL
            Config._app_id = ''
            Config._app_secret = ''
            Config._sign_type = SignType.MD5
            Config._charset = Charset.UTF8
            Config._version = API_VERSION
            Config._client_version = version()
            Config._lang = Lang.EN
            Config._timeout = DEFAULT_TIMEOUT

        return cls._instance

    @classmethod
    def instance(cls, *args, **kwargs):
        if not hasattr(cls, '_instance'):
            cls._instance = cls(*args, **kwargs)

        return cls._instance

    @property
    def api_url(self):
        return Config._api_url

    @api_url.setter
    def api_url(self, value):
        Config._api_url = value if value else API_URL

    @property
    def app_id(self):
        return Config._app_id

    @app_id.setter
    def app_id(self, value):
        Config._app_id = value if value else ''

    @property
    def app_secret(self):
        return Config._app_secret

    @app_secret.setter
    def app_secret(self, value):
        Config._app_secret = value if value else ''

    @property
    def sign_type(self):
        return Config._sign_type.value

    @property
    def charset(self):
        return Config._charset.value

    @property
    def version(self):
        return Config._version

    @property
    def client_version(self):
        return Config._client_version

    @property
    def lang(self):
        return Config._lang.value

    @property
    def timeout(self):
        return Config._timeout

    @timeout.setter
    def timeout(self, value):
        Config._timeout = value



# -*- coding:utf-8 -*-
from enum import Enum, unique


def code_to_message(code):
    """
    {
        "code": 0,
        "message": "success"
    }
    """

    if isinstance(code, int):
        code = Code(code)

    return Message[code]


@unique
class Code(Enum):
    SUCCESS = 0
    SERVER_ERROR = 1
    CLIENT_ERROR = 2
    NETWORK_TIMEOUT = 3

    COMMON_ERROR = 10000
    INVALID_HTTP_METHOD_ERROR = 10001
    NOT_AUTH = 10002
    INVALID_APP_ID_ERROR = 10003
    INVALID_APP_SECRET_ERROR = 10004
    SIGN_ERROR = 10005
    NEED_PRIVILEGE = 10006
    MAX_LIMIT = 10007
    INVALID_METHOD_ERROR = 10008
    INVALID_PARAMS_ERROR = 10009
    REQUIRED_PARAMS_MISSING = 10010
    INVALID_FIELDS_ERROR = 10011
    NOT_FOUND = 10012
    NOT_VIP = 10013


Message = {
    Code.SUCCESS: 'success',
    Code.SERVER_ERROR: 'server error',
    Code.CLIENT_ERROR: 'client error',
    Code.NETWORK_TIMEOUT: 'network timeout',
    Code.COMMON_ERROR: 'common error',
    Code.INVALID_HTTP_METHOD_ERROR: 'invalid http method',
    Code.NOT_AUTH: 'not auth',
    Code.INVALID_APP_ID_ERROR: 'invalid app id',
    Code.INVALID_APP_SECRET_ERROR: 'invalid app secret',
    Code.SIGN_ERROR: 'sign error',
    Code.NEED_PRIVILEGE: 'need privilege',
    Code.MAX_LIMIT: 'max limit',
    Code.INVALID_METHOD_ERROR: 'invalid method',
    Code.INVALID_PARAMS_ERROR: 'invalid params error',
    Code.REQUIRED_PARAMS_MISSING: 'required params missing',
    Code.INVALID_FIELDS_ERROR: 'invalid fields error',
    Code.NOT_FOUND: 'not found',
    Code.NOT_VIP: 'not vip'
}



import os
import time
import json
from datetime import datetime
from uuid import uuid4
import hashlib
import base64


def root_path():
    return os.path.dirname(
        os.path.dirname(os.path.dirname(os.path.abspath(__file__))))


def version():
    fpath = os.path.join(root_path(), 'foxrelax', 'version.txt')
    with open(fpath, 'r') as f:
        f = open(fpath, 'r')
        line = f.readline()
        ver = line.strip()

    return ver


def install_requires():
    fpath = os.path.join(root_path(), 'requirements.txt')
    requires = []

    with open(fpath, 'r') as f:
        for line in f:
            line = line.strip()
            if line and line[0] != '#':
                requires.append(line)

    return requires


def md5(s, mode='hex'):
    """
    计算MD5

    Parameters
    ----------
    s : str
    mode : str
        hex | base64

    Returns
    -------
    str

    """

    m = hashlib.md5()
    if isinstance(s, str):
        m.update(s.encode('utf-8'))
    elif isinstance(s, bytes):
        m.update(s)

    if mode == 'hex':
        result = m.hexdigest()
    elif mode == 'base64':
        result = base64.b64encode(m.digest()).decode('utf-8')

    return result


def gen_timestamp(is_millisecond=True, is_str=True):
    """
    获取当前时间戳

    Parameters
    ----------
    is_millisecond : bool
    is_str : bool

    Examples
    --------
    >>> gen_timestamp(is_millisecond=False, is_str=False)
    1571671574

    >>> gen_timestamp(is_millisecond=True, is_str=False)
    1571671574966

    >>> gen_timestamp(is_millisecond=False, is_str=True)
    2019-10-21 23:26:14

    >>> gen_timestamp(is_millisecond=True, is_str=True)
    2019-10-21 23:26:14.966

    """

    if is_str:
        if is_millisecond:
            ts = datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f')
            ts = ts[:-3]
        else:
            ts = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
    else:
        ts = time.time()
        if is_millisecond:
            ts = int(time.time() * 1000)
        else:
            ts = int(time.time())

    return ts


def gen_uuid():
    """
    返回一个长度为32的UUID

    Parameters
    ----------
    None

    Returns
    -------
    str

    """

    return str(uuid4()).replace('-', '')


def gen_nonce_str():
    """
    返回一个长度为32的nonce str

    Parameters
    ----------
    None

    Returns
    -------
    str

    """

    return gen_uuid()


def sign(req_dict, app_secret, to_md5=True):
    params = []
    for (k, v) in sorted(req_dict.items()):
        if k == 'sign':
            continue

        if not isinstance(v, str):
            v = json.dumps(v, ensure_ascii=False)

        params.append('{0}={1}'.format(k, v))

    # 在最后拼接app_secret
    params.append('app_secret={0}'.format(app_secret))

    # 用&连接各k-v对
    if to_md5:
        return md5('&'.join(params))
    else:
        return '&'.join(params)



# -*- coding:utf-8 -*-
"""
Data APIs
"""

from foxrelax.client import api_client


def echo(text=None, fields=None):
    """
    echo test
    """

    client = api_client()

    return client.query('echo', text=text, fields=fields)


def news_flash(channel=None,
               news_date=None,
               start_date=None,
               end_date=None,
               fields=None):
    """
    新闻快讯
    """

    client = api_client()

    return client.query('news_flash',
                        channel=channel,
                        news_date=news_date,
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


def rise_fall_realtime(fields=None):
    """
    实时涨跌分布
    """

    client = api_client()

    return client.query('rise_fall_realtime', fields=fields)


def limit_rise_fall_realtime(fields=None):
    """
    实时涨跌停板
    """

    client = api_client()

    return client.query('limit_rise_fall_realtime', fields=fields)


def manufacturing_pmi_monthly(trade_date=None,
                              start_date=None,
                              end_date=None,
                              fields=None):
    """
    制造业PMI(月度)
    """

    client = api_client()

    return client.query('manufacturing_pmi_monthly',
                        trade_date=trade_date,
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


def ppi_monthly(trade_date=None, start_date=None, end_date=None, fields=None):
    """
    工业品出厂价格指数(月度)
    """

    client = api_client()

    return client.query('ppi_monthly',
                        trade_date=trade_date,
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


def cci_monthly(trade_date=None, start_date=None, end_date=None, fields=None):
    """
    消费者信心指数(月度)
    """

    client = api_client()

    return client.query('cci_monthly',
                        trade_date=trade_date,
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


def stock_trade_monthly(trade_date=None,
                        start_date=None,
                        end_date=None,
                        fields=None):
    """
    全国股票交易统计(月度)
    """

    client = api_client()

    return client.query('stock_trade_monthly',
                        trade_date=trade_date,
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


def north_top10(trade_date=None,
                exchange=None,
                start_date=None,
                end_date=None,
                fields=None):
    """
    北向资金十大成交股
    """

    client = api_client()

    return client.query('north_top10',
                        trade_date=trade_date,
                        exchange=exchange,
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


def north_money(trade_date=None, start_date=None, end_date=None, fields=None):
    """
    北向资金流向
    """

    client = api_client()

    return client.query('north_money',
                        trade_date=trade_date,
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


def south_money(trade_date=None, start_date=None, end_date=None, fields=None):
    """
    南向资金流向
    """

    client = api_client()

    return client.query('south_money',
                        trade_date=trade_date,
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


def north_money_realtime(fields=None):
    """
    实时北向资金流向
    """

    client = api_client()

    return client.query('north_money_realtime', fields=fields)


def south_money_realtime(fields=None):
    """
    实时南向资金流向
    """

    client = api_client()

    return client.query('south_money_realtime', fields=fields)


def trade_calendar(start_date=None, end_date=None, fields=None):
    """
    交易日历
    """

    client = api_client()

    return client.query('trade_calendar',
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


def area(fields=None):
    """
    地区编码
    """

    client = api_client()

    return client.query('area', fields=fields)


def exchange_info(fields=None):
    """
    交易所信息
    """

    client = api_client()

    return client.query('exchange_info', fields=fields)


def index_info(symbol=None,
               name=None,
               exchange=None,
               publisher=None,
               category=None,
               fields=None):
    """
    指数基本信息
    """

    client = api_client()

    return client.query('index_info',
                        symbol=symbol,
                        name=name,
                        exchange=exchange,
                        publisher=publisher,
                        category=category,
                        fields=fields)


def index_daily(symbol=None,
                trade_date=None,
                start_date=None,
                end_date=None,
                fields=None):
    """
    指数日线行情
    """

    client = api_client()

    return client.query('index_daily',
                        symbol=symbol,
                        trade_date=trade_date,
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


def index_weekly(symbol=None, start_date=None, end_date=None, fields=None):
    """
    指数周线行情
    """

    client = api_client()

    return client.query('index_weekly',
                        symbol=symbol,
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


def index_monthly(symbol=None, start_date=None, end_date=None, fields=None):
    """
    指数月线行情
    """

    client = api_client()

    return client.query('index_monthly',
                        symbol=symbol,
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


def index_bar(symbol=None,
              freq=None,
              adjust=None,
              trade_date=None,
              start_date=None,
              end_date=None,
              fields=None):
    """
    指数通用行情
    """

    client = api_client()

    return client.query('index_bar',
                        symbol=symbol,
                        freq=freq,
                        adjust=adjust,
                        trade_date=trade_date,
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


def stock_info(symbol=None,
               exchange=None,
               market=None,
               list_status=None,
               fields=None):
    """
    股票基本信息
    """

    client = api_client()

    return client.query('stock_info',
                        symbol=symbol,
                        exchange=exchange,
                        market=market,
                        list_status=list_status,
                        fields=fields)


def normalize_symbol(symbols, fields=None):
    """
    格式化股票, 指数symbol
    """

    client = api_client()

    return client.query('normalize_symbol', symbols=symbols, fields=fields)


def normalize_stock_symbol(symbols, fields=None):
    """
    格式化股票symbol
    """

    client = api_client()

    return client.query('normalize_stock_symbol',
                        symbols=symbols,
                        fields=fields)


def normalize_index_symbol(symbols, fields=None):
    """
    格式化指数symbol
    """

    client = api_client()

    return client.query('normalize_index_symbol',
                        symbols=symbols,
                        fields=fields)


def stock_daily(symbol=None,
                trade_date=None,
                start_date=None,
                end_date=None,
                fields=None):
    """
    股票日线行情
    """

    client = api_client()

    return client.query('stock_daily',
                        symbol=symbol,
                        trade_date=trade_date,
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


def stock_weekly(symbol=None, start_date=None, end_date=None, fields=None):
    """
    股票周线行情
    """

    client = api_client()

    return client.query('stock_weekly',
                        symbol=symbol,
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


def stock_monthly(symbol=None, start_date=None, end_date=None, fields=None):
    """
    股票月线行情
    """

    client = api_client()

    return client.query('stock_monthly',
                        symbol=symbol,
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


def valuation(symbol=None, start_date=None, end_date=None, fields=None):
    """
    估值指标
    """

    client = api_client()

    return client.query('valuation',
                        symbol=symbol,
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


def dividend(symbol=None, start_date=None, end_date=None, fields=None):
    """
    分红送股
    """

    client = api_client()

    return client.query('dividend',
                        symbol=symbol,
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


def capital_change(symbol=None, start_date=None, end_date=None, fields=None):
    """
    股本变动
    """

    client = api_client()

    return client.query('capital_change',
                        symbol=symbol,
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


def adj_factor(symbol=None, start_date=None, end_date=None, fields=None):
    """
    股票复权信息
    """

    client = api_client()

    return client.query('adj_factor',
                        symbol=symbol,
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


def stock_bar(symbol=None,
              freq=None,
              adjust=None,
              trade_date=None,
              start_date=None,
              end_date=None,
              fields=None):
    """
    股票通用行情
    """

    client = api_client()

    return client.query('stock_bar',
                        symbol=symbol,
                        freq=freq,
                        adjust=adjust,
                        trade_date=trade_date,
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


# pylint:disable=blacklisted-name
def bar(symbol=None,
        freq=None,
        adjust=None,
        trade_date=None,
        start_date=None,
        end_date=None,
        fields=None):
    """
    通用行情
    """

    client = api_client()

    return client.query('bar',
                        symbol=symbol,
                        freq=freq,
                        adjust=adjust,
                        trade_date=trade_date,
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


def balance_sheet(symbol=None,
                  report_type=None,
                  report_date=None,
                  start_date=None,
                  end_date=None,
                  fields=None):
    """
    股票资产负债表
    """

    client = api_client()

    return client.query('balance_sheet',
                        symbol=symbol,
                        report_type=report_type,
                        report_date=report_date,
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


def income(symbol=None,
           report_type=None,
           report_date=None,
           start_date=None,
           end_date=None,
           fields=None):
    """
    股票利润表
    """

    client = api_client()

    return client.query('income',
                        symbol=symbol,
                        report_type=report_type,
                        report_date=report_date,
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


def cash_flow(symbol=None,
              report_type=None,
              report_date=None,
              start_date=None,
              end_date=None,
              fields=None):
    """
    股票现金流量表
    """

    client = api_client()

    return client.query('cash_flow',
                        symbol=symbol,
                        report_type=report_type,
                        report_date=report_date,
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


def industry_industry(fields=None):
    """
    行业板块
    """

    client = api_client()

    return client.query('industry_industry', fields=fields)


def industry_concept(fields=None):
    """
    概念板块
    """

    client = api_client()

    return client.query('industry_concept', fields=fields)


def industry_region(fields=None):
    """
    地域板块
    """

    client = api_client()

    return client.query('industry_region', fields=fields)


def industry(industry_type=None, fields=None):
    """
    板块(通用)
    """

    client = api_client()

    return client.query('industry', industry_type=industry_type, fields=fields)


def industry_stock_industry(industry_symbol=None, fields=None):
    """
    行业板块成分股
    """

    client = api_client()

    return client.query('industry_stock_industry',
                        industry_symbol=industry_symbol,
                        fields=fields)


def industry_stock_concept(industry_symbol=None, fields=None):
    """
    概念板块成分股
    """

    client = api_client()

    return client.query('industry_stock_concept',
                        industry_symbol=industry_symbol,
                        fields=fields)


def industry_stock_region(industry_symbol=None, fields=None):
    """
    地域板块成分股
    """

    client = api_client()

    return client.query('industry_stock_region',
                        industry_symbol=industry_symbol,
                        fields=fields)


def industry_stock(industry_symbol=None, fields=None):
    """
    板块成分股(通用)
    """

    client = api_client()

    return client.query('industry_stock',
                        industry_symbol=industry_symbol,
                        fields=fields)


def stock_industry_industry(symbol=None, fields=None):
    """
    股票所属行业板块
    """

    client = api_client()

    return client.query('stock_industry_industry',
                        symbol=symbol,
                        fields=fields)


def stock_industry_concept(symbol=None, fields=None):
    """
    股票所属概念板块
    """

    client = api_client()

    return client.query('stock_industry_concept', symbol=symbol, fields=fields)


def stock_industry_region(symbol=None, fields=None):
    """
    股票所属地域板块
    """

    client = api_client()

    return client.query('stock_industry_region', symbol=symbol, fields=fields)


def stock_industry(symbol=None, industry_type=None, fields=None):
    """
    股票所属板块(通用)
    """

    client = api_client()

    return client.query('stock_industry',
                        symbol=symbol,
                        industry_type=industry_type,
                        fields=fields)


def stock_suspend(symbol=None,
                  suspend_date=None,
                  resume_date=None,
                  fields=None):
    """
    股票停复牌信息
    """

    client = api_client()

    return client.query('stock_suspend',
                        symbol=symbol,
                        suspend_date=suspend_date,
                        resume_date=resume_date,
                        fields=fields)


def money_supply(start_date=None, end_date=None, fields=None):
    """
    货币供应量
    """

    client = api_client()

    return client.query('money_supply',
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


def reserve_ratio(trade_date=None,
                  start_date=None,
                  end_date=None,
                  fields=None):
    """
    存款准备金率
    """

    client = api_client()

    return client.query('reserve_ratio',
                        trade_date=trade_date,
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


def gold_foreign_reserve_monthly(trade_date=None,
                                 start_date=None,
                                 end_date=None,
                                 fields=None):
    """
    黄金和外汇储备(月度)
    """

    client = api_client()

    return client.query('gold_foreign_reserve_monthly',
                        trade_date=trade_date,
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


def deposit_rate(start_date=None, end_date=None, fields=None):
    """
    存款利率
    """

    client = api_client()

    return client.query('deposit_rate',
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


def loan_rate(start_date=None, end_date=None, fields=None):
    """
    贷款利率
    """

    client = api_client()

    return client.query('loan_rate',
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


def cpi(start_date=None, end_date=None, fields=None):
    """
    居民消费价格指数(CPI)(全国)
    """

    client = api_client()

    return client.query('cpi',
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)


def cpi_item(item_symbol=None, start_date=None, end_date=None, fields=None):
    """
    居民消费价格指数(CPI)(项目)
    """

    client = api_client()

    return client.query('cpi_item',
                        item_symbol=item_symbol,
                        start_date=start_date,
                        end_date=end_date,
                        fields=fields)



# -*- coding:utf-8 -*-

import json
import pandas as pd
from foxrelax.common.code import Code


class Response:
    def __init__(self, response):
        if isinstance(response, str):
            self._response = response
            response = json.loads(response)

        if 'code' in response:
            self._code = Code(response['code'])
        if 'message' in response:
            self._message = response['message']
        if self._code == Code.SUCCESS and 'data' in response:
            data = response['data']
            fields = data['fields']
            items = data['items']
            df = pd.DataFrame(items, columns=fields)
            self._data = data
            self._result = df
        else:
            self._data = None
            self._result = None
        if 'timestamp' in response:
            self._timestamp = response['timestamp']

    @property
    def code(self):
        return self._code.value

    @property
    def message(self):
        return self._message

    @property
    def data(self):
        return self._data

    @property
    def result(self):
        return self._result

    @property
    def timestamp(self):
        return self._timestamp

    @property
    def is_success(self):
        return self._code == Code.SUCCESS

    def __str__(self):
        return self.__repr__()

    def __repr__(self):
        return self._response



# -*- coding:utf-8 -*-
import os
import io
from setuptools import setup, find_packages

long_desc = """
foxrelax
"""


def root_path():
    return os.path.dirname(os.path.abspath(__file__))


def version():
    fpath = os.path.join(root_path(), 'foxrelax', 'version.txt')
    with open(fpath, 'r') as f:
        f = open(fpath, 'r')
        line = f.readline()
        ver = line.strip()

    return ver


def install_requires():
    fpath = os.path.join(root_path(), 'requirements.txt')
    requires = []

    with open(fpath, 'r') as f:
        for line in f:
            line = line.strip()
            if line and line[0] != '#':
                requires.append(line)

    return requires


setup(
    name='foxrelax',
    version=version(),
    description='foxrelax',
    long_description=io.open('README.md', encoding='utf-8').read(),
    long_description_content_type='text/markdown',
    author='liqiang',
    author_email='liqiang@g.im',
    license='BSD License',
    platforms=['all'],
    url='https://github.com/relaxdl/foxrelax',
    install_requires=install_requires(),
    keywords='foxrelax',
    packages=find_packages(),
    include_package_data=True,
    package_data={'': ['*.txt']},
    classifiers=[
        'Development Status :: 1 - Planning', 'Environment :: Console',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Software Development :: Libraries'
    ],
)



# -*- coding:utf-8 -*-
import json
from foxrelax.config import Config
from foxrelax.common.util import sign, gen_nonce_str, gen_timestamp


class Request:
    def __init__(self, method, params=None):
        self._config = Config()
        self._method = method
        self._params = params if params else {}
        self._app_id = self._config.app_id
        self._app_secret = self._config.app_secret
        self._sign_type = self._config.sign_type
        self._charset = self._config.charset
        self._version = self._config.version
        self._nonce_str = gen_nonce_str()
        self._timestamp = gen_timestamp()
        self._lang = self._config.lang

    @property
    def config(self):
        return self._config

    @property
    def method(self):
        return self._method

    @method.setter
    def method(self, value):
        self._method = value

    @property
    def params(self):
        return self._params

    @params.setter
    def params(self, value):
        self._params = value

    @property
    def app_id(self):
        return self._app_id

    @property
    def app_secret(self):
        return self._app_secret

    @property
    def sign_type(self):
        return self._sign_type

    @property
    def charset(self):
        return self._charset

    @property
    def version(self):
        return self._version

    @property
    def nonce_str(self):
        return self._nonce_str

    @property
    def timestamp(self):
        return self._timestamp

    @property
    def lang(self):
        return self._lang

    def to_dict(self):
        req_dict = {
            'app_id': self.app_id,
            'method': self.method,
            'params': self.params,
            'sign_type': self.sign_type,
            'charset': self.charset,
            'version': self.version,
            'nonce_str': self.nonce_str,
            'timestamp': self.timestamp,
            'lang': self.lang
        }
        req_dict['sign'] = sign(req_dict, self.app_secret)
        return req_dict

    def __str__(self):
        return self.__repr__()

    def __repr__(self):
        return json.dumps(self.to_dict())
